// Copyright (c) 2013 by István Váradi

// This file is part of XPLRA, a remote-access plugin for X-Plane

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

//------------------------------------------------------------------------------

#include <hu/varadiistvan/xplra/xplra.h>

#include "ccommon.h"

#include <stdio.h>
#include <string.h>

#ifdef _WIN32
#include <windows.h>
#else
#include <unistd.h>
#endif

//------------------------------------------------------------------------------

#ifdef _WIN32
void xplra_sleep(int ms)
{
    Sleep(ms);
}
#else
void xplra_sleep(int ms)
{
    usleep(ms*1000);
}
#endif


//------------------------------------------------------------------------------

int main(int argc, char* argv[])
{
    int retval = 0;
    const char* errorString = 0;

    uint16_t codes[] = { 0x0241, 0x0142, 0x0343, 0x0251 };

    printf("Connecting to X-Plane...\n");
    int connectionID = connect_xplane(argc, argv);

    if (connectionID>=0) {
        printf("Connected.\n\n");
    } else {
        printf("Connection failed.\n");
        goto cleanup;
    }


    printf("Registering hotkeys: Ctrl+A, Shift+B, Ctrl+Shift+C, Ctrl+Q...\n");
    if (xplra_register_hotkeys(connectionID,
                               codes, sizeof(codes)/sizeof(uint16_t))<0)
    {
        goto error;
    }
    printf("Registered hotkeys...\n\n");

    printf("Listening to hotkeys...\n");
    while(1) {
        uint8_t states[4];
        char message[256];

        if (xplra_query_hotkeys(connectionID, states, sizeof(states))<0) {
            goto error;
        }

        if (states[3]!=0) {
            if (xplra_unregister_hotkeys(connectionID)<0) {
                goto error;
            }
            break;
        }

        message[0] = '\0';

        if (states[0]!=0) {
            if (message[0]!='\0') strcat(message, ", ");
            strcat(message, "Ctrl+A");
        }
        if (states[1]!=0) {
            if (message[0]!='\0') strcat(message, ", ");
            strcat(message, "Shift+B");
        }
        if (states[2]!=0) {
            if (message[0]!='\0') strcat(message, ", ");
            strcat(message, "Ctrl+Shift+C");
        }

        if (message[0]!='\0') {
            strcat(message, " pressed");
            if (xplra_show_message(connectionID, message, 3.0)<0) {
                goto error;
            }
        }

        xplra_sleep(500);
    }

    goto cleanup;

error:
    errorString = xplra_get_last_error_string(connectionID);
    if (errorString==0) {
        printf("\nUnknown error occured!\n");
    } else {
        printf("\nError: %s\n", errorString);
    }

cleanup:
    if (connectionID>=0) xplra_disconnect(connectionID);
    return retval;
}

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
